﻿using DropBoxClient;
using Ionic.Zip;
using LiteBlog.Common.Components;
using LiteBlog.Common.Enums;
using LiteBlog.Components.Contracts;
using LiteBlog.Components.Enums;
using OAuth2Authorizer;
using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Net;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Text;
using System.Web;

namespace LiteBlog.Components
{
    public class BackupComp : IBackupComp
    {
        private IAuthorizer _authorizer = null;
        public ISettingsComp SettingsComponent { get; set; }
        public IConfigComp ConfigComponent { get; set; }

        public BackupComp()
        {
            _authorizer = AuthorizerBuilder.Create(BuiltInAuthorizers.DropBox);
        }

        public MethodResult Backup()
        {
            var settings = SettingsComponent.Load();
            var accessToken = settings.AccessToken;
            BackupFolder(accessToken, ConfigComponent.DataPath, "Backup.zip");
            BackupFolder(accessToken, ConfigComponent.DocumentPath, "Documents.zip");
            return MethodResult.BackupSuccess;
        }

        private void BackupFolder(string accessToken, string folderName, string zipName)
        {
            Logger.Log(string.Format("Starting to backup {0} ...", folderName));
            using (var zipStream = new MemoryStream())
            {
                using (var zipFile = new Ionic.Zip.ZipFile())
                {
                    zipFile.AddDirectory(folderName);
                    zipFile.Save(zipStream);
                    Logger.Log("Zipped");
                }

                Logger.Log("Hitting dropbox now");
                var client = new OAuth2Client(accessToken);
                client.PutFiles("sandbox", zipName, zipStream);
            }
            Logger.Log(string.Format("Backedup {0}", folderName));
        }

        public string GetRegisterUrl(string redirectUrl, string state)
        {
            var settings = SettingsComponent.Load();
            return _authorizer.GetAuthorizeUrl(settings.AppKey, redirectUrl, state);
        }

        public void GetAndSaveAccessToken(string redirectUrl, string code)
        {
            var settings = SettingsComponent.Load();
            var accessToken = _authorizer.GetAccessToken(settings.AppKey, settings.AppSecret, redirectUrl, code);
            SettingsComponent.Update(SettingType.AccessToken, accessToken.AccessToken);
        }
    }
}
