﻿namespace LiteBlog.Components
{
    using System;
    using System.Web;
    using System.Web.Caching;

    using LiteBlog.Common;
    using LiteBlog.Common.Entities;
    using LiteBlog.Common.Enums;
    using LiteBlog.XmlLayer.Contracts;
    using LiteBlog.Components.Enums;
    using LiteBlog.Components.Contracts;
    using LiteBlog.Components.Helpers;


    public class CacheComp : ICacheComp
    {
        private ICacheContext _context = new CacheContext();

        public T Get<T>(string key) where T : class
        {
            if (HttpContext.Current != null)
            {
                return (T)HttpContext.Current.Cache[key];
            }

            return null;
        }

        public T Get<T>(CacheType type) where T : class
        {
            if (HttpContext.Current != null)
            {
                return (T)HttpContext.Current.Cache[type.ToString()];
            }

            return null;
        }

        public void Put(string key, object value)
        {
            if (HttpContext.Current != null)
            {
                HttpContext.Current.Cache[key] = value;
            }
        }

        public void Put(CacheType type, object value)
        {
            if (HttpContext.Current == null)
                return;

            CacheDependency dep = _context.GetDependency(type);
            if (dep != null)
                HttpContext.Current.Cache.Insert(type.ToString(), value, dep);

            else
                HttpContext.Current.Cache.Insert(type.ToString(), value);

        }

        public void PutPost(string fileID, Post post)
        {
            if (HttpContext.Current == null)
                return;

            CacheDependency dep = _context.GetDependency(CacheType.Post, fileID);
            if (dep != null)
                HttpContext.Current.Cache.Insert(
                    fileID, post, dep, Cache.NoAbsoluteExpiration, new TimeSpan(0, 5, 0));
            else
                HttpContext.Current.Cache.Insert(fileID, post, null, Cache.NoAbsoluteExpiration, new TimeSpan(0, 5, 0));
        }
    }
}