﻿namespace LiteBlog.Components
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using LiteBlog.Common;
    using LiteBlog.Common.Entities;
    using LiteBlog.Components.Enums;
    using LiteBlog.Components.Contracts;
    using LiteBlog.Common.Enums;
    using LiteBlog.XmlLayer.Contracts;
    using LiteBlog.XmlLayer;
    using LiteBlog.Common.Components;

    public class CategoryComp : BaseComp<ICategoryData>, ICategoryComp
    {
        public CategoryComp(ICategoryData data, ICacheComp cache) : base(data, cache) { }
        
        public BlogComp BlogComponent { get; set; }

        public string GetNames(string csvIds)
        {
            string[] ids = csvIds.Split(new[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
            return ids.Select(id => Get(id).Name).Aggregate((cur, next) => cur + ", " + next);
        }

        public List<Category> Get()
        {
            List<Category> categories = _cache.Get<List<Category>>(CacheType.Categories);
            if (categories == null)
            {
                categories = _data.Get().OrderBy(cat => cat.Order).ToList();
                _cache.Put(CacheType.Categories, categories);
            }

            return categories;
        }

        public Category Get(string id)
        {
            return Get().SingleOrDefault(item => item.CatID == id);
        }

        public MethodResult Create(Category item)
        {
            if (IdExists(item.CatID))
                return MethodResult.DuplicateId;
            if (NameExists(item.Name))
                return MethodResult.DuplicateName;
            return _data.Create(item) ? MethodResult.CreateSuccess : MethodResult.DuplicateId;
        }

        [Resolve("BlogComponent")]
        public MethodResult Update(string olid, Category item)
        {
            var oldItem = Get(olid);
            if (oldItem == null)
                return MethodResult.Failure;

            if (oldItem.CatID != item.CatID)
                if (IdExists(item.CatID))
                    return MethodResult.DuplicateId;

            if (oldItem.Name != item.Name)
                if (NameExists(item.Name))
                    return MethodResult.DuplicateName;

            if (olid != item.CatID)
            {
                BlogComponent.ChangeCategory(olid, item.CatID);
            }

            _data.Update(olid, item);
            return MethodResult.UpdateSuccess;
        }

        [Resolve("BlogComponent")]
        public MethodResult Delete(string id)
        {
            BlogComponent.ChangeCategory(id, string.Empty);
            _data.Delete(id);
            return MethodResult.DeleteSuccess;
        }

        private bool IdExists(string id)
        {
            var item = Get().SingleOrDefault(c => c.CatID == id);
            return item != null;
        }

        private bool NameExists(string name)
        {
            var item = Get().SingleOrDefault(c => c.Name == name);
            return item != null;
        }

        internal void Increment(string catId)
        {
            if (!string.IsNullOrEmpty(catId))
                _data.Increment(catId);
        }

        internal void Decrement(string catID)
        {
            if (!string.IsNullOrEmpty(catID))
                _data.Decrement(catID);
        }
    }
}