﻿namespace LiteBlog.Components
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Web;
    using LiteBlog.Common;
    using LiteBlog.Common.Components;
    using LiteBlog.Common.Entities;
    using LiteBlog.Common.Enums;
    using LiteBlog.Components.Contracts;
    using LiteBlog.Components.Enums;
    using LiteBlog.XmlLayer.Contracts;
    using LiteBlog.XmlLayer;

    public class CommentComp : BaseComp<ICommentData>, ICommentComp
    {
        public CommentComp(ICommentData data, ICacheComp cache) : base(data, cache) { }

        public SettingsComp SettingsComponent { get; set; }

        public List<Comment> Get()
        {
            var comments = _cache.Get<List<Comment>>(CacheType.Comments);
            if (comments == null)
            {
                comments = _data.Get();
                _cache.Put(CacheType.Comments, comments);
            }
            return comments;
        }

        public List<Comment> GetUnapproved()
        {
            return Get().Where(c => !c.IsApproved).ToList();
        }

        public List<Comment> GetApproved()
        {
            return Get().Where(c => c.IsApproved).ToList();
        }

        public Comment Get(string id)
        {
            return Get().SingleOrDefault(c => c.ID == id);
        }

        [Resolve("SettingsComponent")]
        public MethodResult Create(Comment comment)
        {
            bool commentModeration = SettingsComponent.Load().CommentModeration;
            comment.Text = comment.RawComment;
            comment.IsApproved = !commentModeration && comment.IsAuthor;
            comment.ID = _data.Create(comment);
            return comment.IsApproved ? MethodResult.CreateSuccess : MethodResult.Pending;
        }

        public MethodResult Update(string olid, Comment comment)
        {
            comment.Text = comment.RawComment; // will process
            _data.Update(comment);
            return MethodResult.UpdateSuccess;
        }

        public MethodResult Delete(string id)
        {
            var comment = Get(id);
            if (comment == null)
                return MethodResult.NotFound;

            _data.Delete(comment);
            return MethodResult.DeleteSuccess;
        }

        public MethodResult Approve(string id)
        {
            _data.Approve(id);
            return MethodResult.ApproveSuccess;
        }

        public MethodResult Reject(string id)
        {
            _data.Reject(id);
            return MethodResult.RejectSuccess;
        }
    }
}