﻿using LiteBlog.Common.Entities;
using LiteBlog.Common.Enums;
using LiteBlog.Components;
using LiteBlog.Components.Contracts;
using LiteBlog.Components.Enums;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.ServiceModel.Syndication;
using System.Text;
using System.Web;
using System.Xml;

namespace LiteBlog.Components
{
    public class FeedComp : IFeedComp
    {
        private ICacheComp _cache;
        
        public FeedComp(ICacheComp cache, IBlogComp blogComp, IPostComp postComp, ICategoryComp catComp, IAuthorComp authorComp, ISettingsComp settingsComp)
        {
            _cache = cache;
            BlogComponent = blogComp;
            PostComponent = postComp;
            CategoryComponent = catComp;
            AuthorComponent = authorComp;
            SettingsComponent = settingsComp;
        }

        public IBlogComp BlogComponent { get; set; }
        public IPostComp PostComponent { get; set; }
        public ICategoryComp CategoryComponent { get; set; }
        public IAuthorComp AuthorComponent { get; set; }
        public ISettingsComp SettingsComponent { get; set; }

        public SyndicationFeed GetFeed()
        {
            var feed = _cache.Get<SyndicationFeed>(CacheType.Feed);
            if (feed == null)
            {
                feed = GetFeedInternal();
                _cache.Put(CacheType.Feed, feed);
            }
            return feed;
        }

        private SyndicationFeed GetFeedInternal()
        {
            
            Settings app = SettingsComponent.Load();
            string url = app.Url;
            if (url == string.Empty)
            {
                Uri uri = System.Web.HttpContext.Current.Request.Url;
                string dnsPath = uri.Scheme + "://" + uri.Host;
                if (!uri.IsDefaultPort)
                {
                    dnsPath = dnsPath + ":" + uri.Port.ToString();
                }

                string appPath = System.Web.HttpContext.Current.Request.ApplicationPath;
                url = dnsPath + appPath;
            }

            if (url.Substring(url.Length - 1, 1) != "/")
            {
                url += "/";
            }

            List<PostInfo> postInfos = BlogComponent.GetPublished();
            List<SyndicationItem> items = new List<SyndicationItem>();

            int count = 0;
            foreach (PostInfo postInfo in postInfos)
            {
                if (++count > 20)
                {
                    break;
                }

                SyndicationItem item = new SyndicationItem();
                item.Title = SyndicationContent.CreatePlaintextContent(postInfo.Title);

                TimeSpan ts = DateTime.UtcNow - DateTime.Now;
                item.PublishDate = new DateTimeOffset(postInfo.Time, ts);

                string url2 = string.Format("{0}Post/{1}", url, postInfo.FileID);
                SyndicationLink link = SyndicationLink.CreateAlternateLink(new Uri(url2));

                item.Links.Add(link);

                var post = PostComponent.Load(postInfo.FileID);
                item.Content = SyndicationContent.CreateHtmlContent(post.Contents);

                foreach (var category in post.Categories)
                    item.Categories.Add(new SyndicationCategory(category.Name));

                items.Add(item);
            }

            var feed = new SyndicationFeed(items);

            feed.Title = SyndicationContent.CreatePlaintextContent(app.Name);
            feed.Links.Add(SyndicationLink.CreateAlternateLink(new Uri(url)));

            List<Author> authors = AuthorComponent.Get();
            foreach (Author author in authors)
            {
                feed.Authors.Add(new SyndicationPerson(author.Email, author.Name, author.Url));
            }

            var catList = CategoryComponent.Get();
            foreach (Category cat in catList)
            {
                feed.Categories.Add(new SyndicationCategory(cat.Name));
            }

            return feed;
        }
    }
}
