﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using LiteBlog.Common;
using LiteBlog.Common.Entities;
using LiteBlog.Common.Enums;
using LiteBlog.Components.Contracts;
using LiteBlog.Components.Enums;
using LiteBlog.XmlLayer;
using LiteBlog.XmlLayer.Contracts;

namespace LiteBlog.Components
{
    public class PageComp : BaseComp<IPageData>, IPageComp
    {
        public PageComp(IPageData data, ICacheComp cache) : base(data, cache) { }

        public List<Page> GetPublished()
        {
            var publishedPages = _cache.Get<List<Page>>(CacheType.Pages);
            if (publishedPages == null)
            {
                var pages = _data.GetPages();
                publishedPages = pages.Where(p => p.Published).ToList();
                _cache.Put(CacheType.Pages, publishedPages);
            }
            return publishedPages;
        }

        public List<Page> Get()
        {
            return _data.GetPages();
        }

        public Page Get(string id)
        {
            var page = GetPublished().SingleOrDefault(p => p.FileId == id);
            if (page == null)
                page = Get().SingleOrDefault(p => p.FileId == id);
            return page;
        }

        public MethodResult Create(Page item)
        {
           _data.Save(item);
            return MethodResult.CreateSuccess;
        }

        public MethodResult Update(string olid, Page item)
        {
            var oldItem = Get(olid);
            if (oldItem.FileId != item.FileId)
            {
                if (Get(item.FileId) != null)
                    return MethodResult.DuplicateId;

                item.Body = oldItem.Body;
                item.Published = oldItem.Published;

                _data.Delete(oldItem.FileId);
                _data.Save(item);
            }
            else
            {
                oldItem.Title = item.Title;
                _data.Save(oldItem);
            }
            return MethodResult.UpdateSuccess;
        }

        public MethodResult Delete(string id)
        {
            _data.Delete(id);
            return MethodResult.DeleteSuccess;
        }

        public MethodResult Publish(Page page)
        {
            string oldFileId = page.FileId;
            Delete(oldFileId);

            string newFileId = page.Title.Replace(" ", "");
            page.FileId = CreateUniqueId(newFileId);
            page.Published = true;
            Create(page);
            return MethodResult.PublishSuccess;
        }

        public MethodResult Unpublish(string id)
        {
            var page = Get(id);
            page.Published = false;
            Save(page);
            return MethodResult.UnpublishSuccess;
        }

        private void Save(Page page)
        {
            Create(page);
        }

        private string CreateUniqueId(string fileId)
        {
            if (_data.IsUniqueId(fileId))
                return fileId;

            int uid = 1;
            string uniqueId = fileId + "-" + uid.ToString();
            while (_data.IsUniqueId(uniqueId))
            {
                ++uid;
                uniqueId = fileId + "-" + uid.ToString();
            }
            return uniqueId;
        }
    }
}