﻿namespace LiteBlog.Controllers
{
    using System;
    using System.Linq;
    using System.Web;
    using System.Web.Mvc;
    using System.Web.Security;
    using LiteBlog.Common;
    using LiteBlog.Components;
    using LiteBlog.Common.Entities;
    using LiteBlog.Components.Contracts;
    using LiteBlog.Components.Enums;
    using System.Web.Script.Serialization;
    using LiteBlog.Common.Enums;
    using LiteBlog.Controllers.Filters;
    using LiteBlog.Models;
    using LiteBlog.Controllers.Helpers;
    using LiteBlog.Common.Components;
    using LiteBlog.Controllers.Contracts;

    public class AdminController : BaseController, IAdminController
    {
        public IPageComp PageComponent { get; set; }
        public IAuthorComp AuthorComponent { get; set; }

        public AdminController(ISettingsComp settingsComp, IPageComp pageComp, IAuthorComp authorComp)
            : base(settingsComp)
        {
            PageComponent = pageComp;
            AuthorComponent = authorComp;
        }

        public ActionResult Menu()
        {
            return PartialView("MenuControl");
        }

        [Resolve("PageComponent")]
        public ActionResult Title()
        {
            var model = PageComponent.GetPublished();
            var settings = _settingsComponent.Load();
            ViewBag.Title = settings.Name;
            ViewBag.UseImage = settings.UseImage;
            ViewBag.ImageUrl = settings.ImageUrl;
            this.ViewData["IsAuthenticated"] = this.User.Identity.IsAuthenticated;
            this.ViewData["UserName"] = this.User.Identity.IsAuthenticated ? this.User.Identity.Name : string.Empty;
            return this.PartialView("TitleControl", model);
        }
       
        [HttpGet]
        [Authorize]
        public ActionResult ChangePassword()
        {
            return View();
        }

        [HttpPost]
        [AjaxExceptionFilter]
        [ValidateFilter]
        [Authorize]
        [Resolve("AuthorComponent")]
        public ActionResult ChangePassword(ChangePasswordModel model)
        {
            var result = AuthorComponent.ValidateUser(model.Name, model.Password);
            if (result == MethodResult.LoginSuccess)
                result = AuthorComponent.ChangePassword(model.Name, model.Password, model.NewPassword);
            var resultModel = GetResultModel(result, model.Name);
            return Json(resultModel);
        }

        [PageExceptionFilter]
        [Resolve("AuthorComponent")]
        public ActionResult Index()
        {
            if (AuthorComponent.Get().Count == 0)
            {
                var author = new Author();
                author.ID = "admin";
                author.Name = "admin";
                author.Email = "admin@liteblog.com";
                author.IsPrimary = true;
                author.Enabled = true;
                AuthorComponent.Create(author);
            }

            ViewBag.ReturnUrl = string.IsNullOrEmpty(Request.QueryString["ReturnUrl"]) ? FormsAuthentication.DefaultUrl : Request.QueryString["ReturnUrl"];
            return this.View();
        }

        [HttpPost]
        [ValidateFilter]
        [AjaxExceptionFilter]
        [Resolve("AuthorComponent")]
        public ActionResult Login(LoginModel model)
        {
            var result = AuthorComponent.ValidateUser(model.Name, model.Password);
            FormsAuthentication.SetAuthCookie(model.Name, false);
            var resultModel = GetResultModel(result, null);
            return Json(resultModel);
        }

        [Authorize]
        public ActionResult Logout()
        {
            FormsAuthentication.SignOut();
            return this.RedirectToAction("Index");
        }
    }
}