﻿namespace LiteBlog.Controllers
{
    using System.Collections.Generic;
    using System.Linq;
    using System.Web.Mvc;
    using LiteBlog.Common;
    using LiteBlog.Components;
    using LiteBlog.Common.Entities;
    using LiteBlog.Components.Contracts;
    using System.Web.Script.Serialization;
    using LiteBlog.Controllers.Filters;
    using LiteBlog.Models;
    using LiteBlog.Controllers.Helpers;
    using LiteBlog.Controllers.Contracts;
    using LiteBlog.Common.Components;

    public class ArchiveController : BaseController, IArchiveController
    {
        private IArchiveComp _archiveComponent = null;
        public IBlogComp BlogComponent { get; set; }
        public IPostComp PostComponent { get; set; }

        public ArchiveController(IArchiveComp archiveComponent, IBlogComp blogComponent, IPostComp postComponent, ISettingsComp settingsComponent) : base(settingsComponent)
        {
            _archiveComponent = archiveComponent;
            BlogComponent = blogComponent;
            PostComponent = postComponent;
        }

        internal IArchiveComp ArchiveComponent
        {
            get { return _archiveComponent; }
        }

        public ActionResult Archive()
        {
            var posts = BlogComponent.GetPublished();
            IEnumerable<IGrouping<int, IGrouping<ArchiveMonth, PostInfo>>> model =
                posts.GroupBy(p => new ArchiveMonth(p.Month, p.Year), ArchiveMonth.Comparer).GroupBy(
                    group => group.Key.Year);

            return View("Archive", GetSiteLayout(), model);
        }

        [ArchiveNotFoundFilter(Order=1)]
        [CrawlerFilterForPosts(Order=2)]
        public ActionResult Index(int year, int month)
        {
            ViewBag.Year = year;
            ViewBag.Month = month;
            ViewBag.MonthName = RouteData.Values["monthName"];

            var postInfos = BlogComponent.GetPostsByMonth(year, month);

            int pageSize = _settingsComponent.Load().PostCount;
            var posts = new List<Post>();
            foreach (var postInfo in postInfos.Take(pageSize))
            {
                var post = PostComponent.Load(postInfo.FileID);
                DisplayHelper.SetDisplayTime(post);
                posts.Add(post);
            }

            // passing posts to CrawlerFilter
            RouteData.Values["posts"] = posts;

            var serializer = new JavaScriptSerializer();
            ViewBag.Json = serializer.Serialize(posts);
            return View("Index", GetSiteLayout(), null);
        }

        [HttpGet]
        public ActionResult GetMore(int year, int month, int page)
        {
            var postInfos = BlogComponent.GetPostsByMonth(year, month);
            int pageSize = _settingsComponent.Load().PostCount;
            int startIndex = page * pageSize;
            var posts = new List<Post>();
            foreach (var postInfo in postInfos.Skip(startIndex).Take(pageSize))
            {
                var post = PostComponent.Load(postInfo.FileID);
                DisplayHelper.SetDisplayTime(post);
                posts.Add(post);
            }
            var resultModel = new ResultModel();
            resultModel.Result = posts.Count > 0 ? true : false;
            var resultModelWithData = new ResultModelWithData<List<Post>>(resultModel, posts);
            return this.Json(resultModelWithData, JsonRequestBehavior.AllowGet);
        }

        public ActionResult Widget()
        {
            List<ArchiveMonth> archMonths = _archiveComponent.Get();
            IEnumerable<IGrouping<int, ArchiveMonth>> model = archMonths.GroupBy(m => m.Year);
            return PartialView(model);
        }
    }
}