﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using LiteBlog.Models;
using LiteBlog.Components;
using LiteBlog.Components.Enums;
using LiteBlog.Controllers.Helpers;
using LiteBlog.Components.Contracts;
using LiteBlog.Common.Components;

namespace LiteBlog.Controllers
{
    public class BaseController : Controller
    {
        protected ISettingsComp _settingsComponent = null;

        public BaseController(ISettingsComp settingsComponent)
        {
            _settingsComponent = settingsComponent;
            ActionInvoker = new CustomActionInvoker();
        }

        public ActionResult PageNotFound()
        {
            return View("PageNotFound");
        }

        public ActionResult ErrorPage(Exception ex)
        {
            return View("Error", ex);
        }

        public ViewResult GetViewResult(string viewName, object model)
        {
            return View(viewName, model);
        }

        public JsonResult GetJsonResult(object model)
        {
            return Json(model);
        }

        protected virtual ResultModel GetResultModel(MethodResult result, params string[] values)
        {
            var resultModel = new ResultModel();
            string message = MessageHelper.GetMessage(GetType().Name, result);
            switch (result)
            {
                case MethodResult.CreateSuccess:
                    resultModel.Result = true;
                    resultModel.Message = string.Format(message, values[0]);
                    break;
                case MethodResult.DuplicateId:
                    resultModel.Result = false;
                    resultModel.Message = string.Format(message, values[0]);
                    break;
                case MethodResult.DuplicateName:
                    resultModel.Result = false;
                    resultModel.Message = string.Format(message, values[1]);
                    break;
                case MethodResult.DuplicateEmail:
                    resultModel.Result = false;
                    resultModel.Message = string.Format(message, values[1]);
                    break;
                case MethodResult.Failure:
                    resultModel.Result = false;
                    resultModel.Message = message;
                    break;
                case MethodResult.UpdateSuccess:
                    resultModel.Result = true;
                    resultModel.Message = string.Format(message, values[0]);
                    break;
                case MethodResult.DeleteSuccess:
                    resultModel.Result = true;
                    resultModel.Message = string.Format(message, values[0]);
                    break;
                case MethodResult.PrimaryRecord:
                    resultModel.Result = false;
                    resultModel.Message = message;
                    break;
                case MethodResult.UnlockSuccess:
                    resultModel.Result = true;
                    resultModel.Message = string.Format(message, values[0]);
                    break;
                case MethodResult.PrimarySuccess:
                    resultModel.Result = true;
                    resultModel.Message = string.Format(message, values[0]);
                    break;
                case MethodResult.EnableSuccess:
                    resultModel.Result = true;
                    resultModel.Message = string.Format(message, values[0]);
                    break;
                case MethodResult.DisableSuccess:
                    resultModel.Result = true;
                    resultModel.Message = string.Format(message, values[0]);
                    break;
                case MethodResult.LoginSuccess:
                    resultModel.Result = true;
                    resultModel.Message = message;
                    break;
                case MethodResult.LoginFailure:
                    resultModel.Result = false;
                    resultModel.Message = message;
                    break;
                case MethodResult.ChangePasswordSuccess:
                    resultModel.Result = true;
                    resultModel.Message = string.Format(message, values[0]);
                    break;
                case MethodResult.SaveSettingsSuccess:
                    resultModel.Result = true;
                    resultModel.Message = message;
                    break;
                case MethodResult.SaveSuccess:
                    resultModel.Result = true;
                    resultModel.Message = string.Format(message, values[0]);
                    break;
                case MethodResult.PublishSuccess:
                    resultModel.Result = true;
                    resultModel.Message = string.Format(message, values[0]);
                    break;
                case MethodResult.UnpublishSuccess:
                    resultModel.Result = true;
                    resultModel.Message = string.Format(message, values[0]);
                    break;
                case MethodResult.NotFound:
                    resultModel.Result = false;
                    resultModel.Message = message;
                    break;
                case MethodResult.Pending:
                    resultModel.Result = true;
                    resultModel.Message = message;
                    break;
                case MethodResult.MultipleDeleteSuccess:
                    resultModel.Result = true;
                    resultModel.Message = message;
                    break;
                case MethodResult.ApproveSuccess:
                    resultModel.Result = true;
                    resultModel.Message = message;
                    break;
                case MethodResult.RejectSuccess:
                    resultModel.Result = true;
                    resultModel.Message = message;
                    break;
                case MethodResult.MultipleApproveSuccess:
                    resultModel.Result = true;
                    resultModel.Message = message;
                    break;
                case MethodResult.MultipleRejectSuccess:
                    resultModel.Result = true;
                    resultModel.Message = message;
                    break;
                case MethodResult.BackupSuccess:
                    resultModel.Result = true;
                    resultModel.Message = message;
                    break;
            }
            return resultModel;

        }

        protected string Skin { get; set; }

        protected string GetSiteLayout()
        {
            return Request.Browser.IsMobileDevice ? "_TabletSiteLayout" : "_SiteLayout";
        }

        protected string GetAdminLayout()
        {
            return Request.Browser.IsMobileDevice ? "_TabletAdminLayout" : "_AdminLayout";
        }
    }
}
