﻿namespace LiteBlog.Controllers
{
    using System;
    using System.Linq;
    using System.Collections.Generic;
    using System.Web.Mvc;
    using LiteBlog.Common;
    using LiteBlog.Models;
    using LiteBlog.Components;
    using LiteBlog.Controllers.Filters;
    using LiteBlog.Common.Entities;
    using System.Web.Script.Serialization;
    using LiteBlog.Common.Enums;
    using LiteBlog.Components.Contracts;
    using LiteBlog.Components.Enums;
    using LiteBlog.Controllers.Helpers;
    using LiteBlog.Controllers.Contracts;


    public class CategoryController : CompController<Category, string>, ICategoryController
    {
        private ICategoryComp _categoryComp = null;
        private IBlogComp _blogComp = null;
        private IPostComp _postComp = null;

        public ICategoryComp CategoryComponent { get { return _categoryComp; } }

        public CategoryController(ICategoryComp categoryComp, IBlogComp blogComp, IPostComp postComp, ISettingsComp settingsComponent) : base(categoryComp, settingsComponent) 
        {
            _categoryComp = categoryComp;
            _blogComp = blogComp;
            _postComp = postComp;
        }

        public override ActionResult Create(Category model)
        {
            return base.Create(model);
        }

        public override ActionResult Update(string olid, Category model)
        {
            return base.Update(olid, model);
        }

        public override ActionResult Delete(string id)
        {
            return base.Delete(id);
        }

        public override ActionResult Manage()
        {
            return base.Manage();
        }

        [HttpPost]
        [Authorize]
        [AjaxExceptionFilter]
        public ActionResult Reorder(string order)
        {
            string[] ids = order.Split(new[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
            for (int idx = 0; idx < ids.Length; idx++)
            {
                var id = ids[idx];
                var category = _component.Get(id);
                category.Order = idx + 1;
                _component.Update(id, category);
            }
            return Json(new ResultModel { Result = true });
        }


        [CategoryNotFoundFilter(Order = 1)]
        [CrawlerFilterForPosts(Order = 2)]
        [PageExceptionFilter]
        public ActionResult Index(string id)
        {
            ViewBag.CategoryId = id;
            ViewBag.CategoryName = RouteData.Values["categoryName"].ToString();

            var postInfos = _blogComp.GetPostsByCategory(id);
            
            int pageSize = _settingsComponent.Load().PostCount;
            var posts = new List<Post>();
            foreach (var postInfo in postInfos.Take(pageSize))
            {
                var post = _postComp.Load(postInfo.FileID);
                DisplayHelper.SetDisplayTime(post);
                posts.Add(post);
            }

            // passing posts to CrawlerFilter
            RouteData.Values["posts"] = posts;

            var serializer = new JavaScriptSerializer();
            ViewBag.Json = serializer.Serialize(posts);
            return View("Index", GetSiteLayout(), null);
        }

        [HttpGet]
        [AjaxExceptionFilter]
        public ActionResult GetMore(string id, int page)
        {
            var postInfos = _blogComp.GetPostsByCategory(id);
            int pageSize = _settingsComponent.Load().PostCount;
            int startIndex = page * pageSize;
            var posts = new List<Post>();
            foreach (var postInfo in postInfos.Skip(startIndex).Take(pageSize))
            {
                var post = _postComp.Load(postInfo.FileID);
                DisplayHelper.SetDisplayTime(post);
                posts.Add(post);
            }
            var resultModel = new ResultModel();
            resultModel.Result = posts.Count > 0 ? true : false;
            var resultModelWithData = new ResultModelWithData<List<Post>>(resultModel, posts);
            return this.Json(resultModelWithData, JsonRequestBehavior.AllowGet);
        }

        [PageExceptionFilter]
        public ActionResult Widget()
        {
            List<Category> categories = _component.Get();
            return PartialView(categories);
        }

        [Authorize]
        [HttpGet]
        [PageExceptionFilter]
        public ActionResult Get()
        {
            return Json(_component.Get(), JsonRequestBehavior.AllowGet);
        }

        protected override string[] GetMessageParams(Category model)
        {
            return new string[] { model.CatID, model.Name };
        }
    }
}