﻿namespace LiteBlog.Controllers
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Web.Mvc;
    using LiteBlog.Common;
    using System.Globalization;
    using LiteBlog.Components;
    using LiteBlog.Common.Components;
    using LiteBlog.Common.Entities;
    using LiteBlog.Components.Contracts;
    using System.Web.Script.Serialization;
    using LiteBlog.Components.Enums;
    using LiteBlog.Models;
    using LiteBlog.Controllers.Filters;
    using LiteBlog.Controllers.Contracts;

    public class CommentController : CompController<Comment, string>, ICommentController
    {
        private ICommentComp _commentComponent = null;

        public CommentController(ICommentComp commentComp, ISettingsComp settingsComp)
            : base(commentComp, settingsComp)
        {
            _commentComponent = commentComp;
            _settingsComponent = settingsComp;
        }


        [Authorize]
        [AjaxExceptionFilter]
        public ActionResult Approve(string id)
        {
            string[] ids = id.Split(new char[] { ',' }, StringSplitOptions.None);
            foreach (var cid in ids)
                _commentComponent.Approve(cid);
            var result = ids.Length > 1 ? MethodResult.MultipleApproveSuccess : MethodResult.ApproveSuccess;
            var resultModel = GetResultModel(result, string.Empty);
            return Json(resultModel, JsonRequestBehavior.AllowGet);
        }

        [Authorize]
        [AjaxExceptionFilter]
        public ActionResult Reject(string id)
        {
            string[] ids = id.Split(new char[] { ',' }, StringSplitOptions.None);
            foreach (var cid in ids)
                _commentComponent.Reject(cid);
            var result = ids.Length > 1 ? MethodResult.MultipleRejectSuccess : MethodResult.RejectSuccess;
            var resultModel = GetResultModel(result, string.Empty);
            return Json(resultModel, JsonRequestBehavior.AllowGet);
        }

        [ValidateInput(false)]
        [AllowAnonymous]
        public override ActionResult Create(Comment comment)
        {
            if (this.User.Identity.IsAuthenticated)
                comment.IsAuthor = true;
            comment.Ip = Request.UserHostAddress;
            comment.Time = DateTimeHelper.Now;
            var result = _component.Create(comment);
            var resultModel = GetResultModel(result, string.Empty);
            var resultModelWithData = new ResultModelWithData<Comment>(resultModel, comment);
            return Json(resultModelWithData, JsonRequestBehavior.AllowGet);
        }

        public override ActionResult Delete(string id)
        {
            string[] ids = id.Split(new char[] {','}, StringSplitOptions.None);
            foreach (var cid in ids)
               _commentComponent.Delete(cid);
            var result = ids.Length > 1 ? MethodResult.MultipleDeleteSuccess : MethodResult.DeleteSuccess;
            var resultModel = GetResultModel(result, string.Empty);
            return Json(resultModel, JsonRequestBehavior.AllowGet);
        }

        [HttpGet]
        public override ActionResult Manage()
        {
            List<Comment> comments = _commentComponent.Get();
            var serializer = new JavaScriptSerializer();
            ViewBag.Json = serializer.Serialize(comments);
            return View("Manage", GetAdminLayout(), null);
        }

        [ValidateInput(false)]
        public override ActionResult Update(string olid, Comment model)
        {
            model.Time = DateTime.ParseExact(model.FormattedTime, ConfigComp.DateFormat, null);
            return base.Update(olid, model);
        }

        public ActionResult Recent()
        {
            var comments = _commentComponent.GetApproved();
            return View("Recent", GetSiteLayout(),comments);
        }

        public ActionResult Widget()
        {
            var comments = _commentComponent.GetApproved().Take(10);
            return PartialView("Widget", comments);
        }
    }
}