﻿namespace LiteBlog.Controllers
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Linq;
    using System.Web.Mvc;
    using System.Web.Routing;
    using System.Xml;

    using LiteBlog.Common;
    using LiteBlog.Components;
    using LiteBlog.Common.Entities;
    using LiteBlog.Common.Components;
    using LiteBlog.Common.Enums;
    using LiteBlog.Components.Contracts;
    using System.Web.Script.Serialization;
    using LiteBlog.Controllers.Helpers;
    using LiteBlog.Controllers.Filters;
    using System.Drawing;
    using LiteBlog.Models;
    using LiteBlog.Controllers.Contracts;

    public class HomeController : BaseController, IHomeController
    {
        private IPageComp _pageComponent = null;
        private IBlogComp _blogComponent = null;
        private IPostComp _postComponent = null;
        private IStatComp _statComponent = null;
        private IFeedComp _feedComponent = null;

        public HomeController(IBlogComp blogComp, IPostComp postComp, IPageComp pageComp, ISettingsComp settingsComp, IStatComp statComp, IFeedComp feedComp) : base(settingsComp)
        {
            _blogComponent = blogComp;
            _postComponent = postComp;
            _settingsComponent = settingsComp;
            _statComponent = statComp;
            _pageComponent = pageComp;
            _feedComponent = feedComp;
        }

        public ActionResult Atom()
        {
            _statComponent.IncrementFeeds();
            var feed = _feedComponent.GetFeed();
            return new FeedActionResult(feed, FeedType.ATOM);
        }

        public ActionResult Rss()
        {
            _statComponent.IncrementFeeds();
            var feed = _feedComponent.GetFeed();
            return new FeedActionResult(feed, FeedType.RSS);
        }

        public ActionResult Error()
        {
            return View("Error", GetSiteLayout(), null);
        }

        [HttpGet]
        [CrawlerFilterForPosts]
        public ActionResult Index()
        {
            var postInfos = _blogComponent.GetDefaultPosts();
            var posts = new List<Post>();
            foreach (var postInfo in postInfos)
            {
                var post = _postComponent.Load(postInfo.FileID);
                DisplayHelper.SetDisplayTime(post);
                posts.Add(post);
            }
            RouteData.Values["posts"] = posts;
            var serializer = new JavaScriptSerializer();
            ViewBag.Json = serializer.Serialize(posts);
            return View("Index", GetSiteLayout(), null);
        }


        public ActionResult Widget()
        {
            _statComponent.IncrementHits();
            var stat = _statComponent.Load();
            return PartialView(stat);
        }

        public ActionResult Statistics()
        {
            var stat = _statComponent.Load();
            return View("Statistics", GetSiteLayout(), stat);
        }

        public ActionResult Title()
        {
            var model = new TitleModel();
            var settings = _settingsComponent.Load();
            ViewBag.Title = model.Title = settings.Name;
            ViewBag.ImageUrl = model.ImageUrl = settings.ImageUrl;
            ViewBag.UseImage = model.UseImage = settings.UseImage;

            var pages = _pageComponent.GetPublished();
            model.Pages = pages;

            model.IsSite = (bool)HttpContext.Application["IsSite"];
            model.Site = (string)HttpContext.Application["Site"];

            model.Skins = new Dictionary<string, Color>();
            foreach (var skin in Enum.GetNames(typeof(Skin)))
            {
                model.Skins[skin] = SkinHelper.GetSkinColor(skin);
            }
            var currentSkin = SkinHelper.GetSkin();
            model.Skin = SkinHelper.GetSkinColor(currentSkin);

            return PartialView("TitleControl", model);
        }
    }
}