﻿using LiteBlog.Common.Entities;
using LiteBlog.Common.Enums;
using LiteBlog.Components.Contracts;
using LiteBlog.Controllers.Contracts;
using LiteBlog.Controllers.Filters;
using LiteBlog.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.Mvc;
using System.Web.Script.Serialization;

namespace LiteBlog.Controllers
{
    public class SettingsController : BaseController, ISettingsController
    {
        private static readonly string StateString = "Any state";
        // Only injection !!!!
        // Property injection through dynamic proxy works only for interface methods!

        public IBackupComp BackupComponent { get; set; }
        public IDocumentComp DocumentComponent { get; set; }

        public SettingsController(ISettingsComp settingsComp, IBackupComp backupComp, IDocumentComp documentComp)
            : base(settingsComp)
        {

            BackupComponent = backupComp;
            DocumentComponent = documentComp;
        }

        [Authorize]
        [HttpGet]
        [PageExceptionFilter]
        public ActionResult Update()
        {
            var settings = _settingsComponent.Load();
            var serializer = new JavaScriptSerializer();
            var timezones = TimeZoneInfo.GetSystemTimeZones().Select(tzi => new ListOptionModel(tzi.Id, tzi.DisplayName));
            var skins = Enum.GetNames(typeof(Skin)).Select(s => new ListOptionModel(s, s));
            var postCountOptions = Enumerable.Range(1, 20).Select(i => new ListOptionModel(i.ToString(), i.ToString()));

            string registerUrl = string.Empty;
            if (string.IsNullOrEmpty(settings.AccessToken))
                registerUrl = BackupComponent.GetRegisterUrl(RedirectUrl, StateString);
            string backupUrl = Url.Action("Backup");

            ViewBag.Json = serializer.Serialize(
                    new
                    {
                        Settings = settings,
                        Timezones = timezones,
                        Skins = skins,
                        PostCountOptions = postCountOptions,
                        RegisterUrl = registerUrl,
                        BackupUrl = backupUrl
                    }
                );
            return View("Update", GetAdminLayout(), null);
        }

        [HttpPost]
        [Authorize]
        [ValidateFilter]
        [AjaxExceptionFilter]
        public ActionResult Update(Settings settings)
        {
            var oldSettings = _settingsComponent.Load();
            settings.AppKey = oldSettings.AppKey;
            settings.AppSecret = oldSettings.AppSecret;
            settings.AccessToken = oldSettings.AccessToken;
            var result = _settingsComponent.Save(settings);
            var resultModel = GetResultModel(result, null);
            return Json(resultModel);
        }

        [HttpPost]
        public ActionResult Upload()
        {
            string imageUrl = string.Empty;
            if (Request.Files != null && Request.Files.Count > 0)
            {
                HttpPostedFileBase file = this.Request.Files[0];

                if (file.ContentLength > 0)
                {
                    var document = DocumentComponent.GetNew(file.FileName);
                    file.SaveAs(document.PhysicalPath);
                    _settingsComponent.Update(SettingType.ImageUrl, document.VirtualPath);
                    imageUrl = VirtualPathUtility.ToAbsolute(document.VirtualPath);
                }
            }
            
            return Json(imageUrl);
        }


        [HttpGet]
        [PageExceptionFilter]
        public ActionResult GetAccessCode(string code, string state)
        {
            if (state == StateString)
            {
                BackupComponent.GetAndSaveAccessToken(RedirectUrl, code);
                return RedirectToAction("Update");
            }

            throw new Exception("Some cross site scripting attack, take care!!!");
        }

        [HttpGet]
        [AjaxExceptionFilter]
        public ActionResult Backup()
        {
            var result = BackupComponent.Backup();
            var resultModel = GetResultModel(result);
            return Json(resultModel, JsonRequestBehavior.AllowGet);
        }

        // This works only in development mode!!!
        // Use https:// for production
        // Also, register the redirect url with dropbox
        private string RedirectUrl
        {
            get
            {
                var urlHelper = new UrlHelper(Request.RequestContext);
                return urlHelper.Action("GetAccessCode", "Settings", null, "http");
            }
        }
        
    }
}
