﻿using LiteBlog.Components;
using LiteBlog.Components.Contracts;
using LiteBlog.Controllers;
using LiteBlog.Controllers.Contracts;
using LiteBlog.XmlLayer;
using LiteBlog.XmlLayer.Contracts;
using StructureMap;
using StructureMap.Configuration.DSL;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace LiteBlog.IoC
{
    // prefer explicit registration (auto-registration not good)
    class IoCRegistry : Registry
    {
        private static readonly string PathArgName = "path";

        internal IoCRegistry()
            : base()
        {
            var dataPath = DataContext.DataPath;
            var objectInterceptor = new ObjectInterceptor();

            // Method interception works only for interfaces
            // Don't intercept concrete classes. (Dynamic proxy supports interception of virtual methods of classes)

            For<IArchiveData>().Use<ArchiveData>().Ctor<string>(PathArgName).Is(dataPath);
            For<IBlogData>().Use<BlogData>().Ctor<string>(PathArgName).Is(dataPath);
            For<ICategoryData>().Use<CategoryData>().Ctor<string>(PathArgName).Is(dataPath);
            For<ICommentData>().Use<CommentData>().Ctor<string>(PathArgName).Is(dataPath);
            For<IPageData>().Use<PageData>().Ctor<string>(PathArgName).Is(dataPath);
            For<IPostData>().Use<PostData>().Ctor<string>(PathArgName).Is(dataPath);
            For<IServiceData>().Use<ServiceData>().Ctor<string>(PathArgName).Is(dataPath);
            For<ISettingsData>().Use<SettingsData>().Ctor<string>(PathArgName).Is(dataPath);
            For<IStatData>().Use<StatData>().Ctor<string>(PathArgName).Is(dataPath);

            For<IConfigComp>().Use<ConfigComp>();
            For<ICacheComp>().Use<CacheComp>();

            For<IArchiveComp>().Use<ArchiveComp>();
            For<ArchiveComp>().Use<ArchiveComp>();

            For<IAuthorComp>().Use<AuthorComp>().InterceptWith(objectInterceptor);
            For<AuthorComp>().Use<AuthorComp>();

            For<IBlogComp>().Use<BlogComp>().Setter<PostComp>().IsTheDefault().InterceptWith(objectInterceptor);
            For<BlogComp>().Use<BlogComp>().Setter<PostComp>().IsTheDefault();

            For<ICategoryComp>().Use<CategoryComp>().InterceptWith(objectInterceptor);
            For<CategoryComp>().Use<CategoryComp>();

            For<ICommentComp>().Use<CommentComp>().InterceptWith(objectInterceptor);
            For<CommentComp>().Use<CommentComp>();

            For<IPageComp>().Use<PageComp>().InterceptWith(objectInterceptor);
            For<PageComp>().Use<PageComp>();

            For<IPostComp>().Use<PostComp>().Setter<CategoryComp>().IsTheDefault().InterceptWith(objectInterceptor);
            For<PostComp>().Use<PostComp>().Setter<CategoryComp>().IsTheDefault();

            For<ISettingsComp>().Use<SettingsComp>();
            For<SettingsComp>().Use<SettingsComp>();

            For<IBackupComp>().Use<BackupComp>().Setter<ISettingsComp>().IsTheDefault().Setter<IConfigComp>().IsTheDefault();

            For<IStatComp>().Use<StatComp>();
            For<IFeedComp>().Use<FeedComp>();
            For<IDocumentComp>().Use<DocumentComp>().Setter<IConfigComp>().IsTheDefault();
            For<DocumentComp>().Use<DocumentComp>().Setter<IConfigComp>().IsTheDefault();

            // Without proxy, dependenceis work well
            For<IAdminController>().Use<AdminController>();
            For<IArchiveController>().Use<ArchiveController>().InterceptWith(objectInterceptor);
            For<IAuthorController>().Use<AuthorController>();
            For<ICategoryController>().Use<CategoryController>();
            For<ICommentController>().Use<CommentController>();
            For<IHomeController>().Use<HomeController>();
            For<IPageController>().Use<PageController>();
            For<IPostController>().Use<PostController>();
            For<ISkinController>().Use<SkinController>();
            For<IScheduleController>().Use<ScheduleController>();
            For<ISettingsController>().Use<SettingsController>();
        }
    }
}
