﻿namespace LiteBlog.XmlLayer
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Xml.Linq;

    using LiteBlog.Common;
    using LiteBlog.Common.Components;
    using LiteBlog.Common.Entities;
    using LiteBlog.XmlLayer.Contracts;
    using LiteBlog.XmlLayer.Helpers;

    public class ArchiveData : IArchiveData
    {
        private static string _path;

        public ArchiveData(string path)
        {
            _path = path + "Archive.xml";
        }


        public static string Path
        {
            get
            {
                return _path;
            }
        }

        public void Increment(ArchiveMonth archiveMonth)
        {
            var root = XElement.Load(_path);
            var refElem = root.Elements("ArchiveMonth").SingleOrDefault(elem => (int)elem.Attribute("Month") == archiveMonth.Month && (int)elem.Attribute("Year") == archiveMonth.Year);
            if (refElem == null)
            {
                archiveMonth.Count = 1;
                Create(archiveMonth);
            }
            else
            {
                int count = (int)refElem.Attribute("Count");
                refElem.SetAttributeValue("Count", ++count);
                XmlHelper.Save(root, _path);
            }
        }

        public void Decrement(ArchiveMonth archiveMonth)
        {
            var root = XElement.Load(_path);
            var refElem = root.Elements("ArchiveMonth").Single(elem => (int)elem.Attribute("Month") == archiveMonth.Month && (int)elem.Attribute("Year") == archiveMonth.Year);
            int count = (int)refElem.Attribute("Count");
            if (--count == 0)
                refElem.Remove();
            else
                refElem.SetAttributeValue("Count", count);
            XmlHelper.Save(root, _path);
        }

        public List<ArchiveMonth> Get()
        {
            var archiveMonths = new List<ArchiveMonth>();
            var root = XElement.Load(_path);
            foreach (var monthElem in root.Elements("ArchiveMonth"))
            {
                var archiveMonth = new ArchiveMonth();
                archiveMonth.Id = (int)monthElem.Attribute("ID");
                archiveMonth.Month = (int)monthElem.Attribute("Month");
                archiveMonth.Year = (int)monthElem.Attribute("Year");
                archiveMonth.Count = (int)monthElem.Attribute("Count");
                archiveMonths.Add(archiveMonth);
            }

            return archiveMonths;
        }

        internal void Create(ArchiveMonth archiveMonth)
        {
            var root = XElement.Load(_path);

            var newElem = new XElement(
                "ArchiveMonth",
                new XAttribute("ID", archiveMonth.Id),
                new XAttribute("Year", archiveMonth.Year),
                new XAttribute("Month", archiveMonth.Month),
                new XAttribute("Count", archiveMonth.Count));

            bool lastNode = true;
            foreach (var refElem in root.Elements("ArchiveMonth"))
            {
                int refId = (int)refElem.Attribute("ID");
                if (archiveMonth.Id > refId)
                {
                    refElem.AddBeforeSelf(newElem);
                    lastNode = false;
                    break;
                }
            }

            if (lastNode)
                root.Add(newElem);

            XmlHelper.Save(root, _path);
        }

    }
}