﻿namespace LiteBlog.XmlLayer
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Xml.Linq;

    using LiteBlog.Common;
    using LiteBlog.Common.Components;
    using LiteBlog.Common.Entities;
    using LiteBlog.XmlLayer.Contracts;
    using LiteBlog.XmlLayer.Helpers;

    public class CategoryData : ICategoryData
    {
        private static string _path;

        public CategoryData(string path)
        {
            _path = path + "Category.xml";
        }

        public static string Path
        {
            get
            {
                return _path;
            }
        }

        public void Increment(string catId)
        {

            var root = XElement.Load(_path);
            var category = root.Elements("Category").Single(elem => (string)elem.Attribute("ID") == catId);
            int count = (int)category.Attribute("Count");
            category.SetAttributeValue("Count", ++count);
            XmlHelper.Save(root, _path);
        }

        public void Decrement(string catId)
        {

            var root = XElement.Load(_path);
            var category = root.Elements("Category").Single(elem => (string)elem.Attribute("ID") == catId);
            int count = (int)category.Attribute("Count");
            category.SetAttributeValue("Count", --count);
            XmlHelper.Save(root, _path);
        }

        public void Delete(string catId)
        {
            var root = XElement.Load(_path);
            var category = root.Elements("Category").Single(elem => (string)elem.Attribute("ID") == catId);
            category.Remove();
            XmlHelper.Save(root, _path);
        }

        public List<Category> Get()
        {
            var categories = new List<Category>();
            var root = XElement.Load(_path);
            foreach (XElement catElem in root.Elements("Category"))
            {
                var cat = new Category();
                cat.CatID = catElem.Attribute("ID").Value;
                cat.Name = catElem.Attribute("Name").Value;
                cat.Count = (int)catElem.Attribute("Count");
                cat.Order = (int)catElem.Attribute("Order");

                categories.Add(cat);
            }
            return categories;
        }

        internal List<Category> GetCategories(string csvIds)
        {
            var filteredCategories = new List<Category>();
            var catIds = csvIds.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
            if (catIds.Length > 0)
            {
                var categories = Get();
                filteredCategories = categories.Where(c => catIds.Contains(c.CatID)).ToList();
            }
            return filteredCategories;
        }

        public bool Create(Category category)
        {
            var root = XElement.Load(_path);
            var categoryElem = root.Elements("Category").SingleOrDefault(elem => (string)elem.Attribute("ID") == category.CatID);
            if (categoryElem != null)
                return false;

            int count = root.Elements("Category").Count<XElement>();

            // Usually order is zero
            if (category.Order == 0 || category.Order > count)
                category.Order = count + 1;

            categoryElem = new XElement(
                    "Category",
                    new XAttribute("ID", category.CatID),
                    new XAttribute("Name", category.Name),
                    new XAttribute("Count", category.Count),
                    new XAttribute("Order", category.Order));

            var refElem = root.Elements("Category").SingleOrDefault(elem => (int)elem.Attribute("Order") > category.Order);
            if (refElem != null)
                refElem.AddBeforeSelf(categoryElem);
            else
                root.Add(categoryElem);

            XmlHelper.Save(root, _path);
            return true;
        }

        public void Update(string id, Category category)
        {
           
           var     root = XElement.Load(_path);
           var categoryElem = root.Elements("Category").SingleOrDefault(elem => (string)elem.Attribute("ID") == id);
            if(categoryElem!=null)
            {
                categoryElem.SetAttributeValue("ID", category.CatID);
                categoryElem.SetAttributeValue("Name", category.Name);
                categoryElem.SetAttributeValue("Order", category.Order);
                // does not update count
                // count must be updated by increment or decrement!
            }
            XmlHelper.Save(root, _path);
        }

    }
}