﻿namespace LiteBlog.XmlLayer
{
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.Linq;
    using System.Xml.Linq;

    using LiteBlog.Common;
    using LiteBlog.Common.Components;
    using LiteBlog.Common.Entities;
    using LiteBlog.Common.Enums;
    using LiteBlog.XmlLayer.Helpers;

    internal class DraftData
    {
        private string _path;
        private static string _dataPath;

        public DraftData(string path)
        {
            _dataPath = path;
            this._path = path + "Draft.xml";
        }

        internal void AddSchedule(string draftId, DateTime publishDate)
        {
            var root = XElement.Load(this._path);
            var draftElem = root.Elements("Draft").Single(elem => (string)elem.Attribute("FileID") == draftId);

            if (draftElem != null)
            {
                draftElem.SetAttributeValue(
                    "Scheduled", publishDate.ToString(DataContext.DateTimeFormat, CultureInfo.InvariantCulture));
            }

            XmlHelper.Save(root, this._path);
        }

        internal void Delete(string draftId)
        {
            var    root = XElement.Load(this._path);
            var draftElem = root.Elements("Draft").SingleOrDefault(elem => (string)elem.Attribute("FileID") == draftId);
            if (draftElem != null)
                draftElem.Remove();
            XmlHelper.Save(root, _path);
        }

        internal string CheckIfDraftExistsForPublishedPost(string fileId)
        {
            var root = XElement.Load(this._path);
            var draftElem = root.Elements("Draft").SingleOrDefault(elem => (string)elem.Attribute("OriginalFileId") == fileId);
            return draftElem!=null ? draftElem.Attribute("FileID").Value : string.Empty;
        }


        internal List<PostInfo> Get()
        {
            var postInfos = new List<PostInfo>();
            var root = XElement.Load(this._path);
            foreach (var draftElem in root.Elements("Draft").Where(elem => elem.Attribute("Scheduled") == null))
            {
                var postInfo = new PostInfo();
                postInfo.Title = draftElem.Attribute("Title").Value;
                postInfo.FileID = draftElem.Attribute("FileID").Value;
                postInfo.Time = DateTime.ParseExact(
                                        draftElem.Attribute("Date").Value, DataContext.DateTimeFormat, CultureInfo.InvariantCulture);
                postInfo.Type = PostType.Draft;
                postInfos.Add(postInfo);
            }

            var scheduled = GetScheduled();
            postInfos.AddRange(scheduled);
            return postInfos;
        }

        internal List<PostInfo> GetScheduled()
        {
            var postInfos = new List<PostInfo>();
            var root = XElement.Load(_path);
            var draftElems = root.Elements("Draft").Where(elem => elem.Attribute("Scheduled") != null);

            foreach (XElement draftElem in draftElems)
            {
                var postInfo = new PostInfo();
                postInfo.Title = draftElem.Attribute("Title").Value;
                postInfo.FileID = draftElem.Attribute("FileID").Value;

                postInfo.Time = DateTime.ParseExact(
                    draftElem.Attribute("Scheduled").Value,
                    DataContext.DateTimeFormat,
                    CultureInfo.InvariantCulture);

                postInfo.Type = PostType.Scheduled;

                postInfos.Add(postInfo);
            }

            return postInfos;
        }

        internal void Insert(string draftId, string fileId, string title, List<Category> categories, DateTime? time = null)
        {
            var root = XElement.Load(this._path);
            if (!time.HasValue)
            {
                time = DateTimeHelper.Now;
            }
            var draftElem = new XElement(
                "Draft",
                new XAttribute("FileID", draftId),
                new XAttribute("OriginalFileId", fileId),
                new XAttribute("Title", title),
                new XAttribute("CatId", GetCsvIds(categories)),
                new XAttribute(
                    "Date",
                    time.Value.ToString(DataContext.DateTimeFormat, CultureInfo.InvariantCulture)));
            root.Add(draftElem);
            XmlHelper.Save(root, _path);
        }

        internal void Update(PostInfo postInfo)
        {
            var root = XElement.Load(_path);
            var draftElem = root.Elements("Draft").SingleOrDefault(elem => (string)elem.Attribute("FileID") == postInfo.FileID);
            if (draftElem != null)
            {
                draftElem.SetAttributeValue("Title", postInfo.Title);
                draftElem.SetAttributeValue("Date", postInfo.Time.ToString(DataContext.DateTimeFormat, CultureInfo.InvariantCulture));
                draftElem.SetAttributeValue("CatId", GetCsvIds(postInfo.Categories));
            }
            XmlHelper.Save(root, _path);
        }

        private string GetCsvIds(List<Category> categories)
        {
            string csvIds = string.Empty;
            if (categories != null && categories.Count > 0)
                csvIds = categories.Select(c => c.CatID).Aggregate((cur, next) => cur + "," + next);
            return csvIds;
        }

    }
}