﻿namespace LiteBlog.XmlLayer
{
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.Linq;
    using System.Xml.Linq;

    using LiteBlog.Common;
    using LiteBlog.Common.Components;
    using LiteBlog.Common.Entities;
    using LiteBlog.XmlLayer.Contracts;
    using LiteBlog.Common.Enums;
    using LiteBlog.XmlLayer.Helpers;

    public class ServiceData : IServiceData
    {
        private static string _path;

        public ServiceData(string path)
        {
            _path = path + "Service.xml";
        }

        public static string Path
        {
            get
            {
                return _path;
            }
        }

        public List<ServiceItem> Load()
        {
            var list = new List<ServiceItem>();
            var root = XElement.Load(_path);

            foreach (XElement svcElem in root.Elements("ServiceItem"))
            {
                ServiceItem item = new ServiceItem();
                item.Id = svcElem.Attribute("ID").Value;
                item.Type = svcElem.Attribute("Type").Value;
                item.Method = svcElem.Attribute("Method").Value;
                try
                {
                    item.LastUpdated = DateTime.ParseExact(
                        svcElem.Attribute("LastUpdated").Value,
                        DataContext.DateTimeFormat,
                        CultureInfo.InvariantCulture);
                }
                catch
                {
                    item.LastUpdated = DateTime.MinValue;
                }

                item.FrequencyType = (FrequencyType)Enum.Parse(typeof(FrequencyType), svcElem.Attribute("Frequency").Value);
                item.Path = _path;
                list.Add(item);
            }

            return list;
        }

        public void Save(List<ServiceItem> svcList)
        {
            var root = XElement.Load(_path);
            foreach (ServiceItem item in svcList)
            {
                var serviceElem = root.Elements("ServiceItem").SingleOrDefault(elem => elem.Attribute("ID").Value == item.Id);
                if (serviceElem != null)
                    serviceElem.SetAttributeValue(
                               "LastUpdated",
                               item.LastUpdated.ToString(DataContext.DateTimeFormat, CultureInfo.InvariantCulture));
            }

            XmlHelper.Save(root, _path);
        }
    }
}
