﻿namespace LiteBlog.XmlLayer
{
    using System;
    using System.Xml.Linq;

    using LiteBlog.Common;
    using LiteBlog.Common.Components;
    using LiteBlog.Common.Entities;
    using LiteBlog.XmlLayer.Contracts;
    using LiteBlog.XmlLayer.Helpers;

    public class SettingsData : ISettingsData
    {
        private static TimeZoneInfo _tzi = null;

        private static string _path;

        public SettingsData(string path)
        {
            _path = path + "Settings.xml";
        }

        public static string Path
        {
            get
            {
                return _path;
            }
        }

        public Settings Load()
        {
            var settings = new Settings();
            var root = XElement.Load(_path);

            settings.Name = (string)root.Element("Name");
            settings.UseImage = (bool)root.Element("UseImage");
            settings.ImageUrl = (string)root.Element("ImageUrl");
            settings.Url = root.Element("Url").Value;
            settings.PostCount = (int)root.Element("PostCount");
            settings.CommentModeration = (bool)root.Element("CommentModeration");
            settings.Timezone = root.Element("Timezone").Value;
            settings.Skin = root.Element("Skin").Value;
            settings.LogFile = root.Element("LogFile").Value;
            settings.LogLevel = root.Element("LogLevel").Value;
            settings.AppKey = root.Element("AppKey").Value;
            settings.AppSecret = root.Element("AppSecret").Value;
            settings.AccessToken = root.Element("AccessToken").Value;
            settings.ShouldBackup = (bool)root.Element("ShouldBackup");
            return settings;
        }

        public void Save(Settings settings)
        {
            var root = XElement.Load(_path);

            var settingsElem = new XElement(
                    "Settings",
                    new XElement("Name", settings.Name),
                    new XElement("UseImage", settings.UseImage),
                    new XElement("ImageUrl", settings.ImageUrl),
                    new XElement("Url", settings.Url),
                    new XElement("PostCount", settings.PostCount),
                    new XElement("Timezone", settings.Timezone),
                    new XElement("CommentModeration", settings.CommentModeration),
                    new XElement("Skin", settings.Skin),
                    new XElement("LogFile", settings.LogFile),
                    new XElement("LogLevel", settings.LogLevel),
                    new XElement("AppKey", settings.AppKey),
                    new XElement("AppSecret", settings.AppSecret),
                    new XElement("AccessToken", settings.AccessToken),
                    new XElement("ShouldBackup", settings.ShouldBackup)
                    );

            root.ReplaceNodes(settingsElem.Elements());
            XmlHelper.Save(root, _path);
        }

        internal TimeZoneInfo TimeZoneInfo
        {
            get
            {
                if (_tzi == null)
                {
                    _tzi = TimeZoneInfo.FindSystemTimeZoneById(GetTimeZoneInfo());
                }

                return _tzi;
            }
        }

        private string GetTimeZoneInfo()
        {
            var settings = Load();
            return settings.Timezone;
        }

    }
}