﻿namespace LiteBlog.XmlLayer
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Xml.Linq;

    using LiteBlog.Common;
    using LiteBlog.Common.Components;
    using LiteBlog.Common.Entities;
    using LiteBlog.XmlLayer.Contracts;
    using LiteBlog.XmlLayer.Helpers;

    public class StatData : IStatData
    {
        private static string _path;
        private static string _dataPath;

        public StatData(string path)
        {
            _path = path + "Stat.xml";
            _dataPath = path;
        }

        public static string Path
        {
            get
            {
                return _path + "Stat.xml";
            }
        }

        public Stat Load()
        {
            var stat = new Stat();
            var root = XElement.Load(_path);
            stat.Path = _path;

            List<XElement> feedElems = new List<XElement>();
            List<XElement> delPostElems = new List<XElement>();

            stat.Visits = int.Parse(root.Element("Summary").Element("Visits").Attribute("Count").Value);
            stat.Hits = int.Parse(root.Element("Summary").Element("Hits").Attribute("Count").Value);
            stat.Feeds = new Dictionary<string, int>();

            foreach (var statElem in root.Element("Feeds").Elements("Stat"))
            {
                string id = statElem.Attribute("id").Value;
                int count = int.Parse(statElem.Attribute("Count").Value);

                if (Stat.IsFeedKey(id))
                {
                    feedElems.Add(statElem);
                    stat.Feeds[id] = count;
                }
            }

            var monthElem = root.Element("MonthlyViews");
            var lastStatElem = monthElem.Elements("Stat").LastOrDefault();
            var thisMonthId = Stat.GetMonthKey(DateTime.Now);

            bool addNew = true;
            if (lastStatElem != null)
            {
                var lastId = lastStatElem.Attribute("id").Value;
                if (lastId == thisMonthId)
                {
                    stat.ViewsThisMonth = int.Parse(lastStatElem.Attribute("Count").Value);
                    addNew = false;
                }
            }

            if (addNew)
            {
                stat.ViewsThisMonth = 0;
                monthElem.Add(new XElement("Stat", new XAttribute("id", thisMonthId), new XAttribute("Count", 0)));
            }

            foreach (XElement feedElem in feedElems)
            {
                feedElem.Remove();
            }

            XmlHelper.Save(root, _path);

            BlogData blogData = new BlogData(_dataPath);
            stat.PageVisits = blogData.GetPublished().OrderByDescending(p => p.Views).ToDictionary(p => p.FileID);

            stat.MonthlyViews = root.Element("MonthlyViews").Elements("Stat").Select(
                    elem => new Tuple<string, int>(Stat.GetMonthFromKey(elem.Attribute("id").Value),
                                                            (int)elem.Attribute("Count")));
            
            return stat;
        }

       
        public void Save(Stat stat)
        {
            var root = XElement.Load(stat.Path);
            root.Element("Summary").Element("Visits").SetAttributeValue("Count", stat.Visits);
            root.Element("Summary").Element("Hits").SetAttributeValue("Count", stat.Hits);
            DateTime now = DateTimeHelper.Now;

            string todayKey = Stat.GetFeedKey(now);
            string yesKey = Stat.GetFeedKey(now.AddDays(-1));
            string twoKey = Stat.GetFeedKey(now.AddDays(-2));

            var feedsElem = root.Element("Feeds");
            foreach (KeyValuePair<string, int> kvp2 in stat.Feeds)
            {
                if (kvp2.Key == todayKey || kvp2.Key == yesKey || kvp2.Key == twoKey)
                {
                    XElement statElem = new XElement(
                        "Stat", new XAttribute("id", kvp2.Key), new XAttribute("Count", kvp2.Value));

                    feedsElem.Add(statElem);
                }
            }

            var monthElem = root.Element("MonthlyViews").Elements("Stat").Last();
            monthElem.SetAttributeValue("Count", stat.ViewsThisMonth);
            XmlHelper.Save(root, stat.Path);

            BlogData blogData = new BlogData(_dataPath);
            blogData.ChangeViews(stat.PageVisits);
        }
    }
}