﻿function Category(p) {
    var self = this;
    this.parent = p;
    this.name = ko.observable().extend({ required: "Please enter url name" });
    this.displayName = ko.observable().extend({ required: "Please enter display name" });
    this.order = ko.observable();
   
    this.isOdd = ko.observable(true);
    this.isEdit = ko.observable(false);
    this.isNew = ko.observable(false);

    this.oldName = ko.observable();
    this.oldDisplayName = ko.observable();
   
    this.load = function (item) {
        this.name(item.CatID);
        this.displayName(item.Name);
        this.order(item.Order);
    }

    this.edit = function () {
        this.isEdit(true);
        this.oldName(this.name());
        this.oldDisplayName(this.displayName());
    }

    this.hasError = ko.computed(function () {
        return this.name.hasError() || this.displayName.hasError();
    }, this);

    this.setError = function () {
        if (this.hasError()) {
            this.parent.errorMessage("Please correct the following errors:");
            if (this.name.hasError())
                this.parent.errors.push(this.name.validationMessage());
            if (this.displayName.hasError())
                this.parent.errors.push(this.displayName.validationMessage());
        } else {
            this.clearError();
        }
    }

    this.clearError = function () {
        this.parent.errors.removeAll();
    }

    this.update = function () {
        if (this.hasError()) {
            this.setError();
        } else {
            var url, input;
            if (this.isEdit() && !this.isNew()) {
                url = appPath + 'Category/Update/'
                input = {
                    olid: this.oldName(),
                    CatID: this.name(),
                    Name: this.displayName()
                };
            } else {
                url = appPath + 'Category/Create/'
                input = {
                    CatID: this.name(),
                    Name: this.displayName()
                };
            }
           
            $.post(url, input, function (data) {
                if (data.Result) {
                    self.parent.redoOdd();
                    self.parent.successMessage(data.Message);
                    self.clearError();
                    setInterval(function () { self.parent.successMessage('') }, settings.fadeOutInterval);
                    self.isEdit(false);
                    if (self.isNew()) {
                        self.isNew(false);
                        self.parent.addEnable(true);
                    }
                } else {
                    self.parent.errorMessage(data.Message);
                    if (data.Errors) {
                        for (var idx = 0; idx < data.Errors.length; idx++)
                            self.parent.errors.push(data.Errors[idx]);
                    }
                }
            });
        }
    }

    this.remove = function () {
        var url = appPath + 'Category/Delete/' + this.name();
        $.get(url, function (data) {
            if (data.Result) {
                self.parent.successMessage(data.Message);
                setInterval(function () { self.parent.successMessage('') }, settings.fadeOutInterval);
                self.clearError();
                self.parent.remove(self);
                self.parent.redoOdd();
            } else {
                self.parent.errorMessage(data.Message);
            }
        });
    }


    this.close = function () {
        this.clearError();
        if (this.isEdit() && !this.isNew()) {
            this.isEdit(false);
            this.name(this.oldName());
            this.displayName(this.oldDisplayName());
        }
        
        if (this.isNew()) {
            this.parent.collection.pop();
            this.parent.addEnable(true);
        }
    }
}


