﻿function ChangePassword() {
    var self = this;
    this.name = ko.observable().extend({ required: "Please enter login name" });
    this.password = ko.observable().extend({ required: "Please enter password" });
    this.newPassword = ko.observable().extend({ required: "Please enter new password" });
    this.repeatPassword = ko.observable().extend({ required: "Please enter repeat password" });

    this.successMessage = ko.observable();
    this.errors = ko.observableArray();
    this.errorMessage = ko.computed({
        read: function () {
            var html = '';
            if (this.errors().length > 0)
                html += this.errors()[0];
            if (this.errors().length > 1) {
                html += "<ul>"
                for (var idx = 1; idx < this.errors().length; idx++)
                    html += "<li>" + this.errors()[idx] + "</li>";
                html += "</ul>";
            }
            return html;
        },
        write: function (value) {
            this.errors.removeAll();
            this.errors.push(value);
        },
        owner: this
    });

    this.hasError = ko.computed(function () {
        var hasError = this.name.hasError() || this.password.hasError() || this.newPassword.hasError() || this.repeatPassword.hasError();
        return hasError || (this.newPassword() != this.repeatPassword());
    }, this);

    this.setError = function () {
        if (this.hasError()) {
            this.errorMessage("Please correct the following errors:");
            if (this.name.hasError())
                this.errors.push(this.name.validationMessage());
            if (this.password.hasError())
                this.errors.push(this.password.validationMessage());
            if (this.newPassword.hasError())
                this.errors.push(this.newPassword.validationMessage());
            if (this.repeatPassword.hasError())
                this.errors.push(this.repeatPassword.validationMessage());
            if (this.repeatPassword()!=this.newPassword())
                this.errors.push('The password entered should match the new password');
        } else {
            this.clearError();
        }
    }

    this.clearError = function () {
        this.errors.removeAll();
    }

    this.changePassword = function () {
        if (this.hasError()) {
            this.setError();
        } else {
            var url = appPath + 'Admin/ChangePassword';
            var input = {
                name: this.name(),
                password: this.password(),
                newPassword: this.newPassword(),
                repeatPassword: this.repeatPassword()
            };
            $.post(url, input, function (data) {
                if (data.Result) {
                    self.errorMessage('');
                    self.successMessage(data.Message);
                    setTimeout(function () { self.successMessage(''); }, settings.fadeOutInterval);
                } else {
                    self.errorMessage(data.Message);
                    if (data.Errors) {
                        for (var idx = 0; idx < data.Errors.length; idx++)
                            self.errors.push(data.Errors[idx]);
                    }
                }
            });
        }
    }
}