﻿function DraftPost() {
    var self = this;
    this.editor = ko.observable();
    this.body = ko.observable();
    this.id = ko.observable();
    this.fileId = ko.observable();
    this.title = ko.observable().extend({ required: "Please enter title" });
    this.time = ko.observable();
    this.type = ko.observable();
    this.published = ko.observable();
    this.author = ko.observable();

    this.categories = ko.observableArray();
    this.selectedCategories = ko.observableArray();
    this.selectedCategoriesJson = ko.computed(function () {
        var jsonArray = new Array();
        for (var idx = 0; idx < this.selectedCategories().length; idx++)
            jsonArray.push("{ \"CatID\": \"" + this.selectedCategories()[idx] + "\" }");
        return "[" + jsonArray.join() + "]";
    }, this);

    this.oldCategories = ko.observableArray();
    this.oldCategoriesJson = ko.computed(function () {
        var jsonArray = new Array();
        for (var idx = 0; idx < this.oldCategories().length; idx++)
            jsonArray.push("{ \"CatID\": \"" + this.oldCategories()[idx] + "\" }");
        return "[" + jsonArray.join() + "]";
    }, this);

    this.attachments = ko.observableArray();
    this.attachmentsJson = ko.computed(function () {
        var jsonArray = new Array();
        for (var idx = 0; idx < this.attachments().length; idx++)
            jsonArray.push(this.attachments()[idx].toJson());
        return "[" + jsonArray.join() + "]";
    }, this);

    this.ie = ko.observable();

    this.successMessage = ko.observable();
    this.errors = ko.observableArray();
    this.errorMessage = ko.computed({
        read: function () {
            var html = '';
            if (this.errors().length > 0)
                html += this.errors()[0];
            if (this.errors().length > 1) {
                html += "<ul>"
                for (var idx = 1; idx < this.errors().length; idx++)
                    html += "<li>" + this.errors()[idx] + "</li>";
                html += "</ul>";
            }
            return html;
        },
        write: function (value) {
            this.errors.removeAll();
            this.errors.push(value);
        },
        owner: this
    });


    this.previewHtml = ko.observable();
    this.isPreview = ko.observable(false);
    this.previewText = ko.computed(function () {
        if (this.isPreview())
            return 'Back to Compose';
        else
            return 'Preview';
    }, this);
    this.previewTextVisible = ko.computed(function () {
        return this.id() ? true : false;
    }, this);

    this.load = function (item) {
        this.id(item.DraftID);
        this.fileId(item.FileID);
        this.title(item.Title);
        this.type(item.Type);
        this.published(item.Type == 2);
        this.time(item.FormattedTime);        
        for (var idx = 0; idx < item.Categories.length; idx++) {
            var category = item.Categories[idx];
            this.selectedCategories.push(category.CatID);
        }
    }

    this.loadJson = function (item) {
        var post = item.Post;
        this.load(post);
        this.body(post.Contents);
        for (var idx = 0; idx < post.Documents.length; idx++) {
            var document = post.Documents[idx];
            this.attachments.push(new Attachment(this.attachments, document.VirtualPath, this.id()));
        }
        for (var idx = 0; idx < post.OldCategories.length; idx++) {
            var oldCategory = post.OldCategories[idx];
            this.oldCategories.push(oldCategory.CatID);
        }
        for (var idx = 0; idx < item.Categories.length; idx++) {
            var category = item.Categories[idx];
            this.categories.push(new option(category.CatID, category.Name));
        }
        setInterval(function () {
            self.autoSave();
        }, 60000);
    }

    this.editLink = ko.computed(function () {
        return appPath + 'Post/Compose/' + this.id();
    }, this);

    this.preview = function () {
        if (!this.isPreview()) {
            var url = appPath + 'Post/Preview/' + this.id();
            $.get(url, function (data) {
                self.previewHtml(data);
                self.isPreview(true);
            });
        } else {
            self.isPreview(false);
        }
    }

    this.hasError = ko.computed(function () {
        return this.title.hasError();
    }, this);

    this.setError = function () {
        if (this.hasError()) {
            self.errorMessage("Please correct the following errors:");
            if (this.title.hasError())
                self.errors.push(this.title.validationMessage());
        } else {
            self.clearError();
        }
    }

    this.clearError = function () {
        self.errors.removeAll();
    }

    this.autoSave = function () {
        if (!this.hasError())
            this.save();
    }

    this.save = function () {
        if (this.hasError()) {
            this.setError();
        } else {
            var url = appPath + 'Post/Save/'
            var input = {
                DraftID: this.id(),
                FileID: this.fileId(),
                Type: this.type(),
                Title: this.title(),
                Contents: this.editor().getData(),
                Time: this.time(),
                Categories: this.selectedCategoriesJson(),
                OldCategories: this.oldCategoriesJson(),
                Documents: this.attachmentsJson()
            };

            $.post(url, input, function (data) {
                if (data.Result) {
                    self.successMessage(data.Message);
                    if (data.Data != '')
                        self.id(data.Data);
                    self.clearError();
                    setInterval(function () { self.successMessage('') }, settings.fadeOutInterval);
                } else {
                    self.errorMessage(data.Message);
                    if (data.Errors) {
                        for (var idx = 0; idx < data.Errors.length; idx++)
                            self.errors.push(data.Errors[idx]);
                    }
                }
            });
        }
    }

    this.publish = function () {
        if (this.hasError()) {
            this.setError();
        } else {
            var url = appPath + 'Post/Publish/'
            var input = {
                DraftID: this.id(),
                FileID: this.fileId(),
                Type: this.type(),
                Title: this.title(),
                Contents: this.editor().getData(),
                Time: this.time(),
                Categories: this.selectedCategoriesJson(),
                OldCategories: this.oldCategoriesJson(),
                Documents: this.attachmentsJson()
            };

            $.post(url, input, function (data) {
                if (data.Result) {
                    self.successMessage(data.Message);
                    self.clearError();
                    window.location = appPath + 'Post/' + data.Data;
                } else {
                    self.errorMessage(data.Message);
                    if (data.Errors) {
                        for (var idx = 0; idx < data.Errors.length; idx++)
                            self.errors.push(data.Errors[idx]);
                    }
                }
            });
        }
    }

    this.close = function () {
        window.location = appPath + 'Post/Manage';
    }
}

function Attachment(parent, path, fileId) {
    var self = this;
    this.parent = parent;
    this.fileId = fileId;
    this.path = ko.observable(path);
    this.id = ko.computed(function () {
        return this.path().substring(this.path().lastIndexOf('/') + 1);
    }, this);

    this.toJson = function () {
        return '{\"virtualPath": \"' + this.path() + '"}';
    }

    this.deleteAttachment = function () {
        var url = appPath + 'Post/DeleteAttachment/' + this.id() + '?fileId=' + (this.fileId ? this.fileId : '');
        $.get(url, function (data) {
            if (data.Result) {
                self.parent.remove(function (item) { return item.id() == self.id() });
            }
        });
    }
}



