﻿function Page(p) {
    var self = this;
    this.parent = p;
    this.editor = ko.observable();
    this.id = ko.observable().extend({ required: "Please enter url name" });
    this.title = ko.observable().extend({ required: "Please enter title" });
    this.body = ko.observable();
    this.published = ko.observable();

    this.successMessage = ko.observable();
    this.errors = ko.observableArray();
    this.errorMessage = ko.computed({
        read: function () {
            var html = '';
            if (this.errors().length > 0)
                html += this.errors()[0];
            if (this.errors().length > 1) {
                html += "<ul>"
                for (var idx = 1; idx < this.errors().length; idx++)
                    html += "<li>" + this.errors()[idx] + "</li>";
                html += "</ul>";
            }
            return html;
        },
        write: function (value) {
            this.errors.removeAll();
            this.errors.push(value);
        },
        owner: this
    });
    

    this.previewHtml = ko.observable();
    this.isPreview = ko.observable(false);

    this.isOdd = ko.observable(true);
    this.isEdit = ko.observable(false);

    this.oldId = ko.observable();
    this.oldTitle = ko.observable();

    this.load = function (item) {
        this.id(item.FileId);
        this.title(item.Title);
        this.body(item.Body);
        this.published(item.Published);
    }

    this.loadJson = function (item) {
        this.load(item);
        // trigger auto-save every minute
        setInterval(function () {
            self.autoSave();
        }, 60000);
    }

    this.edit = function () {
        this.isEdit(true);
        this.oldId(this.id());
        this.oldTitle(this.title());
    }

    this.editLink = ko.computed(function () {
        return appPath + 'Page/Compose/' + this.id();
    }, this);

    this.preview = function () {
        var url = appPath + 'Page/Preview/' + this.id();
        $.get(url, function (data) {
            self.previewHtml(data);
            self.isPreview(true);
        });
    }

    this.closePreview = function () {
        this.isPreview(false);
    }

    this.hasError = ko.computed(function () {
        return this.id.hasError() || this.title.hasError();
    }, this);

    this.hasComposeError = ko.computed(function () {
        return this.title.hasError();
    }, this);

    this.setError = function () {
        if (this.hasError()) {
            self.parent.errorMessage("Please correct the following errors:");
            if (this.id.hasError())
                self.parent.errors.push(this.id.validationMessage());
            if (this.title.hasError())
                self.parent.errors.push(this.title.validationMessage());
        } else {
            this.clearError();
        }
    }

    this.setComposeError = function () {
        if (this.hasComposeError()) {
            self.errorMessage("Please correct the following errors:");
            if (this.title.hasError())
                self.errors.push(this.title.validationMessage());
        } else {
            self.clearComposeError();
        }
    }

    this.clearError = function () {
        self.parent.errors.removeAll();
    }

    this.clearComposeError = function () {
        self.errors.removeAll();
    }

    this.update = function () {
        if (this.hasError()) {
            this.setError();
        } else {
            var url = appPath + 'Page/Update/'
            var input = {
                olid: this.oldId(),
                FileId: this.id(),
                Title: this.title()
            };

            $.post(url, input, function (data) {
                if (data.Result) {
                    self.parent.redoOdd();
                    self.parent.successMessage(data.Message);
                    self.clearError();
                    setInterval(function () { self.parent.successMessage('') }, settings.fadeOutInterval);
                    self.isEdit(false);
                } else {
                    self.parent.errorMessage(data.Message);
                    if (data.Errors) {
                        for (var idx = 0; idx < data.Errors.length; idx++)
                            self.parent.errors.push(data.Errors[idx]);
                    }
                }
            });
        }
    }

    this.remove = function () {
        var url = appPath + 'Page/Delete/' + this.id();
        $.get(url, function (data) {
            if (data.Result) {
                self.parent.successMessage(data.Message);
                setInterval(function () { self.parent.successMessage('') }, settings.fadeOutInterval);
                self.clearError();
                self.parent.remove(self);
                self.parent.redoOdd();
            } else {
                self.parent.errorMessage(data.Message);
            }
        });
    }

    this.unpublish = function () {
        var url = appPath + 'Page/Unpublish/' + this.id();
        var input = {
            FileId: this.id(),
            Title: this.title(),
            Body: this.body(),
            Published: this.published()
        };
        $.post(url, input, function (data) {
            if (data.Result) {
                self.parent.successMessage(data.Message);
                setInterval(function () { self.parent.successMessage('') }, settings.fadeOutInterval);
                self.clearError();
                self.published(false);
            } else {
                self.parent.errorMessage(data.Message);
            }
        });
    }


    this.close = function () {
        this.clearError();
        if (this.isEdit()) {
            this.isEdit(false);
            this.id(this.oldId());
            this.title(this.oldTitle());
        }
    }

    this.autoSave = function () {
        if (!this.hasComposeError())
            this.save();
    }

    this.save = function () {
        if (this.hasComposeError()) {
            this.setComposeError();
        } else {
            var url = appPath + 'Page/Save/'
            var input = {
                FileId: this.id(),
                Title: this.title(),
                Body: this.editor().getData(),
                Published: this.published()
            };

            $.post(url, input, function (data) {
                if (data.Result) {
                    self.successMessage(data.Message);
                    self.id(data.Data);
                    self.clearComposeError();
                    setInterval(function () { self.successMessage('') }, settings.fadeOutInterval);
                } else {
                    self.errorMessage(data.Message);
                    if (data.Errors) {
                        for (var idx = 0; idx < data.Errors.length; idx++)
                            self.errors.push(data.Errors[idx]);
                    }
                }
            });
        }
    }

    this.publish = function () {
        if (this.hasComposeError()) {
            this.setComposeError();
        } else {
            var url = appPath + 'Page/Publish/'
            var input = {
                FileId: this.id(),
                Title: this.title(),
                Body: this.editor().getData(),
                Published: this.published()
            };

            $.post(url, input, function (data) {
                if (data.Result) {
                    self.successMessage(data.Message);
                    self.clearComposeError();
                    setInterval(function () { self.closePage(); }, settings.fadeOutInterval);
                } else {
                    self.errorMessage(data.Message);
                    if (data.Errors) {
                        for (var idx = 0; idx < data.Errors.length; idx++)
                            self.errors.push(data.Errors[idx]);
                    }
                }
            });
        }
    }

    this.closePage = function () {
        window.location = appPath + 'Page/Manage';
    }
}


